@@ -82,6 +82,8 @@ class ParseTree(object):
 			for supervised classification.
 		:vartype available_measure_functions: int
 		"""
+		if not (0.0 < delta < 1.0):
+			raise ValueError("delta must be in (0,1)")
 		self.delta = delta
 		self.regime = regime
 		self.sub_regime = sub_regime
@@ -293,7 +295,6 @@ class ParseTree(object):
 				ast_node.func.id not in self.available_measure_functions
 				):
 			if len(ast_node.args) == 0: 
-				
 				raise RuntimeError(
 					"Please check the syntax of the function: "
 				   f" {new_node.name}()."
@@ -303,6 +304,16 @@ class ParseTree(object):
 					"Please check the syntax of the function:"
 				   f" {new_node.name}()."
 				   " It appears you provided more than two arguments")
+			if ast_node.func.id in ['abs','exp'] and len(ast_node.args) > 1:
+				raise RuntimeError(
+					"Please check the syntax of the function:"
+				   f" {new_node.name}()."
+				   " It appears you provided more than one argument")
+			if ast_node.func.id in ['min','max'] and len(ast_node.args) == 1:
+				raise RuntimeError(
+					"Please check the syntax of the function: "
+				   f"{new_node.name}(). "
+				   "This function must take two arguments.")
 			for ii,arg in enumerate(ast_node.args):
 				if ii == 0:
 					new_node.left = self._ast_tree_helper(arg)
@@ -623,11 +634,6 @@ class ParseTree(object):
 					kwargs['data_dict'] = data_dict
 					kwargs['datasize'] = datasize
 				
-				# Check if user specified a bound_method for this node
-				if 'base_node_bounding_dict' in kwargs:
-					if node.name in kwargs['base_node_bounding_dict']:
-						self.base_node_dict[node.name]['bound_method'] = \
-							kwargs['base_node_bounding_dict'][node.name]
 				bound_method = self.base_node_dict[node.name]['bound_method']
 				bound_result = node.calculate_bounds(
 					bound_method=bound_method,
