@@ -8,7 +8,9 @@ from seldonian.dataset import (DataSetLoader,
 	SupervisedDataSet)
 from seldonian.safety_test.safety_test import SafetyTest
 from seldonian.utils.io_utils import load_json
+from seldonian.utils.tutorial_utils import generate_data
 from seldonian.models.models import LinearRegressionModel
+from seldonian.RL.RL_model import RL_model
 
 
 two_interval_options = [
@@ -518,18 +520,75 @@ def test_math_functions():
 			 " It appears you provided more than two arguments")
 	assert str(excinfo.value) == error_str
 
-	# constraint_str = 'abs((PR | [X]), (PR | [Y]))'
-	# delta = 0.05
-	# pt = ParseTree(delta,
-	# 	regime='supervised_learning',
-	# 	sub_regime='classification',
-	# 	columns=['X','Y','Z'])
-	# # with pytest.raises(RuntimeError) as excinfo:
-	# pt.create_from_ast(constraint_str)
+
+	constraint_str = 'abs((PR | [X]), (PR | [Y]))'
+	delta = 0.05
+	pt = ParseTree(delta,
+		regime='supervised_learning',
+		sub_regime='classification',
+		columns=['X','Y','Z'])
+	with pytest.raises(RuntimeError) as excinfo:
+		pt.create_from_ast(constraint_str)
 	
-	# error_str = ("Please check the syntax of the function: min()."
-	# 		 " It appears you provided more than two arguments")
-	# assert str(excinfo.value) == error_str
+	error_str = ("Please check the syntax of the function: "
+				f"abs(). "
+				"It appears you provided more than one argument")
+	assert str(excinfo.value) == error_str
+
+	constraint_str = 'exp((PR | [X]), (PR | [Y]))'
+	delta = 0.05
+	pt = ParseTree(delta,
+		regime='supervised_learning',
+		sub_regime='classification',
+		columns=['X','Y','Z'])
+	with pytest.raises(RuntimeError) as excinfo:
+		pt.create_from_ast(constraint_str)
+	
+	error_str = ("Please check the syntax of the function: "
+				f"exp(). "
+				"It appears you provided more than one argument")
+	assert str(excinfo.value) == error_str
+
+	constraint_str = 'max((PR | [X]))'
+	delta = 0.05
+	pt = ParseTree(delta,
+		regime='supervised_learning',
+		sub_regime='classification',
+		columns=['X','Y','Z'])
+	with pytest.raises(RuntimeError) as excinfo:
+		pt.create_from_ast(constraint_str)
+	
+	error_str = ("Please check the syntax of the function: "
+				f"max(). "
+				"This function must take two arguments.")
+	assert str(excinfo.value) == error_str
+
+	constraint_str = 'min((PR | [X]))'
+	delta = 0.05
+	pt = ParseTree(delta,
+		regime='supervised_learning',
+		sub_regime='classification',
+		columns=['X','Y','Z'])
+	with pytest.raises(RuntimeError) as excinfo:
+		pt.create_from_ast(constraint_str)
+	
+	error_str = ("Please check the syntax of the function: "
+				f"min(). "
+				"This function must take two arguments.")
+	assert str(excinfo.value) == error_str
+
+	constraint_str = 'abs()'
+	delta = 0.05
+	pt = ParseTree(delta,
+		regime='supervised_learning',
+		sub_regime='classification',
+		columns=['X','Y','Z'])
+	with pytest.raises(RuntimeError) as excinfo:
+		pt.create_from_ast(constraint_str)
+	
+	error_str = ("Please check the syntax of the function:  abs()."
+			" It appears you provided no arguments")
+	assert str(excinfo.value) == error_str
 
 def test_measure_functions_recognized():
 	delta = 0.05
@@ -605,6 +664,15 @@ def test_measure_function_with_conditional_bad_syntax_captured():
 			pt.create_from_ast(constraint_str)
 		
 		assert str(excinfo.value) == error_str
+
+	constraint_str = "(Mean_Error | [G])"
+	pt = ParseTree(delta,regime='supervised_learning',
+			sub_regime='regression')
+	with pytest.raises(RuntimeError) as excinfo:
+		pt.create_from_ast(constraint_str)
+	error_str = ("A column provided in your constraint str: G " 
+		"was not in the list of  columns provided: []")
+	assert str(excinfo.value) == error_str	
  
 def test_measure_function_from_wrong_regime():
 	""" Test that if a measure function from the incorrect 
@@ -737,6 +805,25 @@ def test_unary_op():
 	assert pt.n_base_nodes == 1
 	assert len(pt.base_node_dict) == 1
 
+	constraint_str = '+Mean_Error'
+	pt = ParseTree(delta,regime='supervised_learning',
+		sub_regime='regression',columns=['M'])
+	with pytest.raises(NotImplementedError) as excinfo:
+		pt.create_from_ast(constraint_str)
+	error_str = ("Error parsing your expression."
+					" A unary operator was used which we do not support: "
+					f"+")
+	assert str(excinfo.value) == error_str
+	# assert pt.root.name == 'sub'
+	# assert pt.root.right.value == 10
+	# assert pt.root.left.name == 'mult'
+	# assert pt.root.left.left.value == -1
+	# assert pt.root.left.right.name == 'abs'
+	# assert pt.root.left.right.left.name == 'Mean_Error | [M]'
+	# assert pt.n_nodes == 6
+	# assert pt.n_base_nodes == 1
+	# assert len(pt.base_node_dict) == 1
+
 def test_raise_error_on_excluded_operators():
 
 	constraint_str = 'FPR^4'
@@ -813,6 +900,44 @@ def test_multiple_conditional_columns_assigned():
 	assert len(pt.base_node_dict) == 1  
 	assert pt.root.left.left.conditional_columns == ['X','Y','Z']
 
+def test_math_functions_propagate():
+	np.random.seed(0)
+	data_pth = 'static/datasets/supervised/GPA/gpa_classification_dataset.csv'
+	metadata_pth = 'static/datasets/supervised/GPA/metadata_classification.json'
+
+	metadata_dict = load_json(metadata_pth)
+
+	loader = DataSetLoader(
+		regime='supervised_learning')
+
+	dataset = loader.load_supervised_dataset(
+		filename=data_pth,
+		metadata_filename=metadata_pth,
+		include_sensitive_columns=False,
+		include_intercept_term=True,
+		file_type='csv')
+
+	model_instance = LinearRegressionModel()
+
+	constraint_str = 'exp(FPR - 0.5)'
+	delta = 0.05
+	pt = ParseTree(delta,regime='supervised_learning',
+		sub_regime='classification',
+		columns=dataset.df.columns)
+	
+	pt.create_from_ast(constraint_str)
+	pt.assign_deltas(weight_method='equal')
+
+	# propagate the bounds with example theta value
+	# theta = np.hstack([np.array([0.0,0.0]),np.random.uniform(-0.05,0.05,10)])
+	theta = np.random.uniform(-0.05,0.05,10)
+	pt.propagate_bounds(theta=theta,dataset=dataset,
+		model=model_instance,branch='safety_test',
+		regime='supervised_learning')
+	assert pt.root.lower == pytest.approx(0.5990300)
+	assert pt.root.upper == pytest.approx(0.5999346)
+
+
 def test_deltas_assigned_equally():
 	constraint_str = 'abs((Mean_Error|[M]) - (Mean_Error|[F])) - 0.1'
 	delta = 0.05 
@@ -997,7 +1122,7 @@ def test_duplicate_base_nodes():
 	pt.propagate_bounds()
 	assert pt.base_node_dict['FPR']['bound_computed'] == True
 
-def test_ttest_bound(generate_data):
+def test_ttest_bound():
 	# dummy data for linear regression
 	np.random.seed(0)
 	numPoints=1000
@@ -1092,7 +1217,7 @@ def test_ttest_bound(generate_data):
 	# assert pt.root.lower == float('-inf') # not computed
 	assert pt.root.upper == pytest.approx(-0.930726)
 
-def test_bad_bound_method(generate_data):
+def test_bad_bound_method():
 	# dummy data for linear regression
 	np.random.seed(0)
 	numPoints=1000
@@ -1232,7 +1357,9 @@ def test_bad_bound_method(generate_data):
 	error_str = (f"Bounding method {bound_method} is not supported")
 	assert str(excinfo.value) == error_str
 
-def test_evaluate_constraint(generate_data,gpa_classification_dataset):
+def test_evaluate_constraint(
+	gpa_classification_dataset,
+	RL_gridworld_dataset):
 	# Evaluate constraint mean, not the bound
 	# test all of the statistics in all regimes
 
@@ -1274,29 +1401,8 @@ def test_evaluate_constraint(generate_data,gpa_classification_dataset):
 
 	assert pt.root.value == pytest.approx(-1.06248)
 
-	# Mean error
-	constraint_str = 'Mean_Error - 2.0'
-	delta = 0.05 
-
-	pt = ParseTree(delta,regime='supervised_learning',
-		sub_regime='regression')
-	pt.create_from_ast(constraint_str)
-
-	pt.assign_deltas(weight_method='equal')
-	pt.assign_bounds_needed()
-	assert pt.n_nodes == 3
-	assert pt.n_base_nodes == 1
-	assert len(pt.base_node_dict) == 1
-	
-	theta = np.array([0,1])
-	pt.evaluate_constraint(theta=theta,dataset=dataset,
-		model=model_instance,regime='supervised_learning',
-		branch='safety_test')
-
-	assert pt.root.value == pytest.approx(-2.013617)
-
 	### Classification
-	constraint_str = '(PR + NR + FPR + FNR + TPR + TNR + logistic_loss) - 10.0'
+	constraint_str = '(abs(PR) + exp(NR*2) + FPR/4.0 + max(FNR,TPR) + min(TNR,logistic_loss)) - 10.0'
 	constraint_strs = [constraint_str]
 	deltas = [0.05]
 
@@ -1310,7 +1416,35 @@ def test_evaluate_constraint(generate_data,gpa_classification_dataset):
 	pt.evaluate_constraint(theta=theta,dataset=dataset,
 		model=model,regime='supervised_learning',
 		branch='safety_test')
-	assert pt.root.value == pytest.approx(-6.306852)
+	print(pt.root.value)
+	assert pt.root.value == pytest.approx(-5.656718)
+
+	### RL
+	constraint_str = 'J_pi_new >= -0.25'
+	constraint_strs = [constraint_str]
+	deltas = [0.05]
+	parse_trees = make_parse_trees_from_constraints(
+		constraint_strs,
+	    deltas,
+	    regime='reinforcement_learning',
+	    sub_regime='all',
+	    columns=[],
+	    delta_weight_method='equal')
+
+	(dataset,policy,
+		env_kwargs,primary_objective) = RL_gridworld_dataset()
+				
+	frac_data_in_safety = 0.6
+
+	# Model
+
+	model = RL_model(policy=policy,env_kwargs=env_kwargs)
+	theta_init = model.policy.get_params()
+	pt = parse_trees[0]
+	pt.evaluate_constraint(theta=theta_init,dataset=dataset,
+		model=model,regime='reinforcement_learning',
+		branch='safety_test')
+	assert pt.root.value == pytest.approx(0.0440292)
 	
 def test_reset_parse_tree():
 	
@@ -1394,7 +1528,7 @@ def test_single_conditional_columns_propagated():
 
 	assert len(pt.base_node_dict["Mean_Error | [M]"]['data_dict']['features']) == 22335
 	pt.reset_base_node_dict()
-	
+
 def test_build_tree():
 	""" Test the convenience function that builds the tree,
 	weights deltas, and assigns bounds all in one """
@@ -1431,3 +1565,72 @@ def test_build_tree():
 	assert pt2.base_node_dict['FNR']['upper'] == float('inf')
 	assert pt2.base_node_dict['FNR']['bound_computed'] == False
 
+def test_bad_delta():
+	""" Test that supplying delta not in (0,1) raises a ValueError
+	"""
+	constraint_str = 'FPR <= 0.1'
+	delta = 0.0
+	with pytest.raises(ValueError) as excinfo:
+		pt = ParseTree(
+			delta,
+			regime='supervised_learning',
+			sub_regime='classification')	
+
+	error_str = ("delta must be in (0,1)")
+	assert str(excinfo.value) == error_str
+
+	delta = 1.0
+	with pytest.raises(ValueError) as excinfo:
+		pt = ParseTree(
+			delta,
+			regime='supervised_learning',
+			sub_regime='classification')	
+
+	error_str = ("delta must be in (0,1)")
+	assert str(excinfo.value) == error_str
+	
+	delta = -2.5
+	with pytest.raises(ValueError) as excinfo:
+		pt = ParseTree(
+			delta,
+			regime='supervised_learning',
+			sub_regime='classification')	
+
+	error_str = ("delta must be in (0,1)")
+	assert str(excinfo.value) == error_str
+
+	delta = 7
+	with pytest.raises(ValueError) as excinfo:
+		pt = ParseTree(
+			delta,
+			regime='supervised_learning',
+			sub_regime='classification')	
+
+	error_str = ("delta must be in (0,1)")
+	assert str(excinfo.value) == error_str
+	
+
+def test_e_assigned_as_constant_node():
+	
+	constraint_str = 'FPR <= e*0.05'
+	delta = 0.05 
+
+	pt = ParseTree(delta,regime='supervised_learning',
+		sub_regime='classification')
+	pt.create_from_ast(constraint_str)
+	assert pt.root.right.left.name == 'e'
+	assert isinstance(pt.root.right.left,ConstantNode)
+
+def test_base_node_bounding_dict():
+	
+	constraint_str = 'FPR <= 0.25'
+	delta = 0.05 
+
+	pt = ParseTree(delta,regime='supervised_learning',
+		sub_regime='classification')
+	# Fill out tree
+	pt.build_tree(
+		constraint_str=constraint_str,
+		delta_weight_method='equal')
+
+	
