@@ -2,7 +2,7 @@ import pytest
 import autograd.numpy as np
 
 from seldonian.utils.stats_utils import (stddev,
-	tinv,weighted_sum_gamma,generate_data)
+	tinv,weighted_sum_gamma)
 
 ### Begin tests
 
@@ -48,25 +48,4 @@ def test_weighted_sum_gamma():
 	assert weighted_sum_gamma(arr,gamma=0.9) == float('inf')
 
 	arr=[float('-inf'),float('inf')]
-	assert np.isnan(weighted_sum_gamma(arr,gamma=0.9))
-
-def test_generate_data():
-	""" Test the function used for generating synthetic 
-	data for the tutorial example """
-	np.random.seed(42)
-	numPoints = 1000
-	X,Y = generate_data(numPoints)
-	assert X.mean() == pytest.approx(0.01933206)
-	assert Y.mean() == pytest.approx(0.09016829)
-	assert X.std() == pytest.approx(0.978726208)
-	assert Y.std() == pytest.approx(1.368570511)
-
-	assert len(X) == numPoints
-	assert len(Y) == numPoints
-
-def test_interactive(monkeypatch):
-	monkeypatch.setattr('builtins.input', lambda _: "Mark")
-
-	# go about using input() like you normally would:
-	i = input("What is your name?")
-	assert i == "Mark"
+	assert np.isnan(weighted_sum_gamma(arr,gamma=0.9))
\ No newline at end of file
